import unittest

from dwca.terms import DWCLanguage
from xml_common.utils import Language


class RecordLevelTest(unittest.TestCase):

    def test_format(self):
        language_field = DWCLanguage(1)
        self.assertEqual(language_field.format("en"), Language.ENG)
        self.assertEqual(language_field.format("english"), Language.ENG)
        self.assertEqual(language_field.format("English"), Language.ENG)
        self.assertEqual(language_field.format("ENG"), Language.ENG)
        self.assertEqual(language_field.format("eng"), Language.ENG)
        self.assertEqual(language_field.format("es"), Language.SPA)
        self.assertEqual(language_field.format("spanish"), Language.SPA)
        self.assertEqual(language_field.format("Spanish"), Language.SPA)
        self.assertEqual(language_field.format("SPA"), Language.SPA)
        self.assertEqual(language_field.format("spa"), Language.SPA)
        self.assertEqual(language_field.format("sdgdfgd"), Language.UND)

    def test_unformat(self):
        language_field = DWCLanguage(1)
        self.assertEqual(language_field.unformat(Language.ENG), "eng")
        self.assertEqual(language_field.unformat(Language.SPA), "spa")
        self.assertEqual(language_field.unformat(Language.CAL), "cal")

    def test_unformat_two_letters(self):
        language_field = DWCLanguage(1, two_letter_coding=True)
        self.assertEqual(language_field.unformat(Language.ENG), "en")
        self.assertEqual(language_field.unformat(Language.SPA), "es")
        with self.assertRaisesRegex(ValueError, "does not have a two-letters"):
            language_field.unformat(Language.CAL)
            language_field.unformat(Language.UND)


if __name__ == '__main__':
    unittest.main()



import unittest

from xml_common.utils import Language


class LanguageTest(unittest.TestCase):
    def test_word(self):
        self.assertEqual(Language.get_language("Spanish"), Language.SPA)
        self.assertEqual(Language.get_language("English"), Language.ENG)
        self.assertEqual(Language.get_language("Carolinian"), Language.CAL)
        self.assertEqual(Language.get_language("Undetermined"), Language.UND)

    def test_word_cap(self):
        self.assertEqual(Language.get_language("SPANISH"), Language.SPA)
        self.assertEqual(Language.get_language("ENGLISH"), Language.ENG)
        self.assertEqual(Language.get_language("CAROLINIAN"), Language.CAL)
        self.assertEqual(Language.get_language("UNDETERMINED"), Language.UND)

    def test_word_lower(self):
        self.assertEqual(Language.get_language("spanish"), Language.SPA)
        self.assertEqual(Language.get_language("english"), Language.ENG)
        self.assertEqual(Language.get_language("carolinian"), Language.CAL)
        self.assertEqual(Language.get_language("undetermined"), Language.UND)

    def test_two_letter(self):
        self.assertEqual(Language.get_language("es"), Language.SPA)
        self.assertEqual(Language.get_language("en"), Language.ENG)
        self.assertEqual(Language.get_language("mk"), Language.MKD)

    def test_three_letter(self):
        self.assertEqual(Language.get_language("spa"), Language.SPA)
        self.assertEqual(Language.get_language("eng"), Language.ENG)
        self.assertEqual(Language.get_language("mkd"), Language.MKD)
        self.assertEqual(Language.get_language("und"), Language.UND)

    def test_three_letter_cap(self):
        self.assertEqual(Language.get_language("SPA"), Language.SPA)
        self.assertEqual(Language.get_language("ENG"), Language.ENG)
        self.assertEqual(Language.get_language("CAL"), Language.CAL)
        self.assertEqual(Language.get_language("UND"), Language.UND)

    def test_special(self):
        self.assertEqual(Language.get_language(""), Language.UND)
        with self.assertRaisesRegex(NotImplementedError, "dfsgsdfg language not implemented yet"):
            Language.get_language("dfsgsdfg")

    def test_two_letters_return(self):
        self.assertEqual("es", Language.SPA.two_letters)
        self.assertEqual("en", Language.ENG.two_letters)
        with self.assertRaisesRegex(ValueError, "does not have a two-letters"):
            Language.CAL.two_letters


if __name__ == '__main__':
    unittest.main()



from dwca.terms import Field


class IsMarine(Field):
    """
    A boolean flag indicating whether the taxon is a marine organism, i.e. can be found in/above sea water.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isMarine"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class IsFreshwater(Field):
    """
    A boolean flag indicating whether the taxon occurrs in freshwater habitats, i.e. can be found in/above rivers or lakes.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isFreshwater"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class IsTerrestrial(Field):
    """
    A boolean flag indicating the taxon is a terrestial organism, i.e. occurrs on land as opposed to the sea.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isTerrestrial"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class IsInvasive(Field):
    """
    Flag indicating a species known to be invasive/alien in some are of the world. detailed native and introduced distribution areas can be published with the distribution extension..

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isInvasive"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class IsHybrid(Field):
    """
    Flag indicating a hybrid organism. this does not have to be reflected in the name, but can be based on other studies like chromosome numbers etc.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isHybrid"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class IsExtinct(Field):
    """
    Flag indicating an extinct organism. details about the timeperiod the organism has lived in can be supplied below.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/isExtinct"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class LivingPeriod(Field):
    """
    The (geological) time a currently extinct organism is known to have lived. for geological times of fossils ideally based on a vocabulary like http://en.wikipedia.org/wiki/geologic_column.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/livingPeriod"
    TYPE = bool

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class AgeInDays(Field):
    """
    Maximum observed age of an organism given as number of days.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/ageInDays"
    TYPE = int

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class SizeInMillimeters(Field):
    """
    Maximum observed size of an organism in millimeter. can be either height, length or width, whichever is greater..

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/sizeInMillimeters"
    TYPE = float

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class MassInGrams(Field):
    """
    Maximum observed weight of an organism in grams..

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/massInGrams"
    TYPE = float

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class LifeForm(Field):
    """
    A term describing the growth/lifeform of an organism. should be based on a vocabulary like raunkiær for plants: http://en.wikipedia.org/wiki/raunkiær_plant_life-form. recommended vocabulary: http://rs.gbif.org/vocabulary/gbif/life_form.xml.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/lifeForm"
    TYPE = str

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return

class SpeciesSex(Field):
    """
    Comma seperated list of known sexes to exist for this organism. recommended vocabulary is: http://rs.gbif.org/vocabulary/gbif/sex.xml.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.tdwg.org/dwc/terms/sex"
    TYPE = str

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return



from dwca.terms import Field


class ThreatStatus(Field):
    """
    Threat status of a species as defined by iucn: https://www.iucnredlist.org/resources/categories-and-criteria.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://iucn.org/terms/threatStatus"
    TYPE = str

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return


class AppendixCITES(Field):
    """
    The cites (convention on international trade in endangered species of wild fauna and flora) appendix number the taxa is listed. it is possible to have different appendix numbers for different areas, but "global" as an area is also valid if its the same worldwide.

    Parameters
    ----------
    index : int | str
        Specifies the position of the column in the row.
    default: TYPE, optional
        Specifies a value to use if one is not supplied.
    vocabulary: str, optional
        An URI for a vocabulary that the source values for this Field are based on.
    """
    URI = "http://rs.gbif.org/terms/1.0/appendixCITES"
    TYPE = str

    def __init__(self, index: int | str, default: TYPE = None, vocabulary: str = None) -> None:
        super().__init__(index, default, vocabulary)
        return



import unittest

from xml_common.utils import CamelCaseEnum


class TestCamelCaseEnum(unittest.TestCase):
    class ExampleCamelCaseEnum(CamelCaseEnum):
        ONE_VALUE = 1
        TWO_VALUE = 2
        THREE_VALUE = 3
        NOCONVERSIONVALUE = 4

    def test_to_camel_case(self):
        self.assertEqual(
            "oneValue",
            TestCamelCaseEnum.ExampleCamelCaseEnum.ONE_VALUE.to_camel_case(),
            "Wrong conversion to camel case (oneValue)."
        )
        self.assertEqual(
            "twoValue",
            TestCamelCaseEnum.ExampleCamelCaseEnum.TWO_VALUE.to_camel_case(),
            "Wrong conversion to camel case (twoValue)."
        )
        self.assertEqual(
            "threeValue",
            TestCamelCaseEnum.ExampleCamelCaseEnum.THREE_VALUE.to_camel_case(),
            "Wrong conversion to camel case (threeValue)."
        )
        self.assertEqual(
            "noconversionvalue",
            TestCamelCaseEnum.ExampleCamelCaseEnum.NOCONVERSIONVALUE.to_camel_case(),
            "Wrong conversion to camel case (noconversionvalue)."
        )

    def test_to_str(self):
        self.assertEqual(
            "One Value",
            str(TestCamelCaseEnum.ExampleCamelCaseEnum.ONE_VALUE),
            "Wrong conversion to str (One Value)."
        )
        self.assertEqual(
            "Two Value",
            str(TestCamelCaseEnum.ExampleCamelCaseEnum.TWO_VALUE),
            "Wrong conversion to str (Two Value)."
        )
        self.assertEqual(
            "Three Value",
            str(TestCamelCaseEnum.ExampleCamelCaseEnum.THREE_VALUE),
            "Wrong conversion to str (Three Value)."
        )
        self.assertEqual(
            "Noconversionvalue",
            str(TestCamelCaseEnum.ExampleCamelCaseEnum.NOCONVERSIONVALUE),
            "Wrong conversion to str (noconversionvalue)."
        )

    def test_get_enum(self):
        self.assertEqual(
            TestCamelCaseEnum.ExampleCamelCaseEnum.ONE_VALUE,
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("oneValue"),
            "Wrong parse one Value."
        )
        self.assertEqual(
            TestCamelCaseEnum.ExampleCamelCaseEnum.TWO_VALUE,
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("twoValue"),
            "Wrong parse two Value."
        )
        self.assertEqual(
            TestCamelCaseEnum.ExampleCamelCaseEnum.THREE_VALUE,
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("threeValue"),
            "Wrong parse three Value."
        )
        self.assertEqual(
            TestCamelCaseEnum.ExampleCamelCaseEnum.NOCONVERSIONVALUE,
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("noconversionvalue"),
            "Wrong parse noconversionvalue."
        )
        with self.assertRaisesRegex(ValueError, "No Conversion Value", msg="No Conversion Value parsed anyway."):
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("No Conversion Value")
        with self.assertRaisesRegex(ValueError, "fourValue", msg="Not present value parsed anyway."):
            TestCamelCaseEnum.ExampleCamelCaseEnum.get_enum("fourValue")


if __name__ == '__main__':
    unittest.main()



import unittest

from xml_common.utils import read_string


class TestReadFile(unittest.TestCase):
    def test_read_string(self):
        self.assertEqual(
            "This is a string",
            read_string("This is a string"),
            "String parsed correctly."
        )

    def test_read_bytes(self):
        self.assertEqual(
            "This is a bytes",
            read_string(b"This is a bytes"),
            "Bytes parsed correctly."
        )

    def test_read_with_encoding(self):
        an_string = "This is a string to be passed as ascii"
        as_ansi = an_string.encode("ascii")
        self.assertEqual(
            an_string,
            read_string(as_ansi, encoding="ascii"),
            "String as ansi parsed correctly."
        )


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

from enum import Enum


TWO_LETTERS = {
    "en": "ENG",
    "ja": "JPN",
    "es": "SPA",
    "it": "ITA",
    "ru": "RUS",
    "pl": "POL",
    "fi": "FIN",
    "pt": "POR",
    "ca": "CAT",
    "tr": "TUR",
    "no": "NOR",
    "sv": "SWE",
    "nb": "NOB",
    "nn": "NNO",
    "co": "COS",
    "hu": "HUN",
    "af": "AFR",
    "hr": "HRV",
    "sl": "SLV",
    "gl": "GLG",
    "gd": "GLA",
    "da": "DAN",
    "et": "EST",
    "ar": "ARA",
    "he": "HEB",
    "sr": "SRP",
    "mt": "MLT",
    "mg": "MLG",
    "vi": "VIE",
    "wo": "WOL",
    "tl": "TGL",
    "ko": "KOR",
    "fo": "FAO",
    "sm": "SMO",
    "so": "SOM",
    "km": "KHM",
    "te": "TEL",
    "ta": "TAM",
    "id": "IND",
    "ty": "TAH",
    "sw": "SWA",
    "dv": "DIV",
    "ml": "MAL",
    "th": "THA",
    "jv": "JAV",
    "mr": "MAR",
    "gu": "GUJ",
    "to": "TON",
    "fj": "FIJ",
    "sd": "SND",
    "kn": "KAN",
    "si": "SIN",
    "mh": "MAH",
    "bg": "BUL",
    "kl": "KAL",
    "gv": "GLV",
    "iu": "IKU",
    "bn": "BEN",
    "ch": "CHA",
    "lt": "LIT",
    "lv": "LAV",
    "ky": "KIR",
    "am": "AMH",
    "uk": "UKR",
    "ve": "VEN",
    "ks": "KAS",
    "ne": "NEP",
    "cr": "CRE",
    "ee": "EWE",
    "or": "ORI",
    "eo": "EPO",
    "de": "DEU",
    "nl": "NLD",
    "fr": "FRA",
    "el": "ELL",
    "cs": "CES",
    "sq": "SQI",
    "ka": "KAT",
    "zh": "ZHO",
    "eu": "EUS",
    "ro": "RON",
    "sk": "SLK",
    "is": "ISL",
    "mi": "MRI",
    "cy": "CYM",
    "ms": "MSA",
    "fa": "FAS",
    "my": "MYA",
    "mk": "MKD",
}


class Language(Enum):
    """
    The language of the resource.

    Attributes
    ----------
    ENG : str
        English language.
    JPN : str
        Japanese language.
    SPA : str
        Spanish language.
    ITA : str
        Italian language.
    RUS : str
        Russian language.
    POL : str
        Polish language.
    FIN : str
        Finnish language.
    POR : str
        Portuguese language.
    CAT : str
        Catalan language.
    TUR : str
        Turkish language.
    NOR : str
        Norwegian language.
    SWE : str
        Swedish language.
    NOB : str
        Bokmål, Norwegian language.
    NNO : str
        Norwegian Nynorsk language.
    COS : str
        Corsican language.
    HUN : str
        Hungarian language.
    AFR : str
        Afrikaans language.
    HRV : str
        Croatian language.
    SLV : str
        Slovenian language.
    GLG : str
        Galician language.
    GLA : str
        Gaelic language.
    DAN : str
        Danish language.
    EST : str
        Estonian language.
    ARA : str
        Arabic language.
    SCO : str
        Scots language.
    HEB : str
        Hebrew language.
    SRP : str
        Serbian language.
    MLT : str
        Maltese language.
    MLG : str
        Malagasy language.
    VIE : str
        Vietnamese language.
    WOL : str
        Wolof language.
    HAW : str
        Hawaiian language.
    TGL : str
        Tagalog language.
    ROM : str
        Romany language.
    PAP : str
        Papiamento language.
    KOR : str
        Korean language.
    FAO : str
        Faroese language.
    SMO : str
        Samoan language.
    SOM : str
        Somali language.
    KHM : str
        Central Khmer language.
    TEL : str
        Telugu language.
    TAM : str
        Tamil language.
    IND : str
        Indonesian language.
    TAH : str
        Tahitian language.
    NIU : str
        Niuean language.
    SWA : str
        Swahili language.
    SRN : str
        Sranan Tongo language.
    DIV : str
        Divehi language.
    TKL : str
        Tokelau language.
    MAL : str
        Malayalam language.
    THA : str
        Thai language.
    JAV : str
        Javanese language.
    MAR : str
        Marathi language.
    GUJ : str
        Gujarati language.
    BIK : str
        Bikol language.
    TSI : str
        Tsimshian language.
    TON : str
        Tonga (Tonga Islands) language.
    FIJ : str
        Fijian language.
    PAU : str
        Palauan language.
    ACE : str
        Achinese language.
    PAG : str
        Pangasinan language.
    ILO : str
        Iloko language.
    WAR : str
        Waray language.
    SND : str
        Sindhi language.
    PAM : str
        Pampanga language.
    CEB : str
        Cebuano language.
    KAN : str
        Kannada language.
    HAI : str
        Haida language.
    SIN : str
        Sinhala language.
    FON : str
        Fon language.
    FAN : str
        Fang language.
    MAH : str
        Marshallese language.
    TVL : str
        Tuvalu language.
    BUL : str
        Bulgarian language.
    KAL : str
        Kalaallisut language.
    GLV : str
        Manx language.
    IKU : str
        Inuktitut language.
    BEN : str
        Bengali language.
    HIL : str
        Hiligaynon language.
    MEN : str
        Mende language.
    CHA : str
        Chamorro language.
    LIT : str
        Lithuanian language.
    LAV : str
        Latvian language.
    KIR : str
        Kirghiz language.
    AMH : str
        Amharic language.
    UKR : str
        Ukrainian language.
    VEN : str
        Venda language.
    KAS : str
        Kashmiri language.
    RAP : str
        Rapanui language.
    NEP : str
        Nepali language.
    KOK : str
        Konkani language.
    CRE : str
        Cree language.
    KOS : str
        Kosraean language.
    SUS : str
        Susu language.
    EWE : str
        Ewe language.
    ORI : str
        Oriya language.
    EPO : str
        Esperanto language.
    DEU : str
        German language.
    NLD : str
        Dutch language.
    FRA : str
        French language.
    ELL : str
        Greek, Modern (1453-) language.
    CES : str
        Czech language.
    SQI : str
        Albanian language.
    KAT : str
        Georgian language.
    ZHO : str
        Chinese language.
    EUS : str
        Basque language.
    RON : str
        Romanian language.
    SLK : str
        Slovak language.
    ISL : str
        Icelandic language.
    MRI : str
        Maori language.
    CYM : str
        Welsh language.
    MSA : str
        Malay language.
    FAS : str
        Persian language.
    MYA : str
        Burmese language.
    MKD : str
        Macedonian language.
    MEY : str
        Hassaniyya language.
    CMN : str
        Mandarin Chinese language.
    DNJ : str
        Dan language.
    BCN : str
        Bali (Nigeria) language.
    VIF : str
        Vili language.
    NLG : str
        Gela language.
    CAL : str
        Carolinian language.
    UND : str
        Undetermined code for unknown languages.
    """
    ENG = "English"
    JPN = "Japanese"
    SPA = "Spanish"
    ITA = "Italian"
    RUS = "Russian"
    POL = "Polish"
    FIN = "Finnish"
    POR = "Portuguese"
    CAT = "Catalan"
    TUR = "Turkish"
    NOR = "Norwegian"
    SWE = "Swedish"
    NOB = "Bokmål, Norwegian"
    NNO = "Norwegian Nynorsk"
    COS = "Corsican"
    HUN = "Hungarian"
    AFR = "Afrikaans"
    HRV = "Croatian"
    SLV = "Slovenian"
    GLG = "Galician"
    GLA = "Gaelic"
    DAN = "Danish"
    EST = "Estonian"
    ARA = "Arabic"
    SCO = "Scots"
    HEB = "Hebrew"
    SRP = "Serbian"
    MLT = "Maltese"
    MLG = "Malagasy"
    VIE = "Vietnamese"
    WOL = "Wolof"
    HAW = "Hawaiian"
    TGL = "Tagalog"
    ROM = "Romany"
    PAP = "Papiamento"
    KOR = "Korean"
    FAO = "Faroese"
    SMO = "Samoan"
    SOM = "Somali"
    KHM = "Central Khmer"
    TEL = "Telugu"
    TAM = "Tamil"
    IND = "Indonesian"
    TAH = "Tahitian"
    NIU = "Niuean"
    SWA = "Swahili"
    SRN = "Sranan Tongo"
    DIV = "Divehi"
    TKL = "Tokelau"
    MAL = "Malayalam"
    THA = "Thai"
    JAV = "Javanese"
    MAR = "Marathi"
    GUJ = "Gujarati"
    BIK = "Bikol"
    TSI = "Tsimshian"
    TON = "Tonga (Tonga Islands)"
    FIJ = "Fijian"
    PAU = "Palauan"
    ACE = "Achinese"
    PAG = "Pangasinan"
    ILO = "Iloko"
    WAR = "Waray"
    SND = "Sindhi"
    PAM = "Pampanga"
    CEB = "Cebuano"
    KAN = "Kannada"
    HAI = "Haida"
    SIN = "Sinhala"
    FON = "Fon"
    FAN = "Fang"
    MAH = "Marshallese"
    TVL = "Tuvalu"
    BUL = "Bulgarian"
    KAL = "Kalaallisut"
    GLV = "Manx"
    IKU = "Inuktitut"
    BEN = "Bengali"
    HIL = "Hiligaynon"
    MEN = "Mende"
    CHA = "Chamorro"
    LIT = "Lithuanian"
    LAV = "Latvian"
    KIR = "Kirghiz"
    AMH = "Amharic"
    UKR = "Ukrainian"
    VEN = "Venda"
    KAS = "Kashmiri"
    RAP = "Rapanui"
    NEP = "Nepali"
    KOK = "Konkani"
    CRE = "Cree"
    KOS = "Kosraean"
    SUS = "Susu"
    EWE = "Ewe"
    ORI = "Oriya"
    EPO = "Esperanto"
    DEU = "German"
    NLD = "Dutch"
    FRA = "French"
    ELL = "Greek, Modern (1453-)"
    CES = "Czech"
    SQI = "Albanian"
    KAT = "Georgian"
    ZHO = "Chinese"
    EUS = "Basque"
    RON = "Romanian"
    SLK = "Slovak"
    ISL = "Icelandic"
    MRI = "Maori"
    CYM = "Welsh"
    MSA = "Malay"
    FAS = "Persian"
    MYA = "Burmese"
    MKD = "Macedonian"
    MEY = "Hassaniyya"
    CMN = "Mandarin Chinese"
    DNJ = "Dan"
    BCN = "Bali (Nigeria)"
    VIF = "Vili"
    NLG = "Gela"
    CAL = "Carolinian"
    UND = "Undetermined"

    @staticmethod
    def get_language(abbreviation: str) -> Language:
        if abbreviation == "":
            return Language.UND
        for lang in Language:
            try:
                if lang.name.lower() == TWO_LETTERS[abbreviation.lower()].lower():
                    return lang
            except KeyError:
                if lang.name.lower() == abbreviation.lower():
                    return lang
                elif lang.value.lower() == abbreviation.lower():
                    return lang
        raise NotImplementedError(f"{abbreviation} language not implemented yet")

    @property
    def two_letters(self):
        for key, value in TWO_LETTERS.items():
            if value == self.name:
                return key
        raise ValueError(f"{repr(self)} does not have a two-letters abbreviation in ISO 639-1")
